@@ -28,6 +28,8 @@
 #' @template param_heartbeat_expire
 #' @template param_seed
 #' @template param_data_format
+#' @template param_message_log
+#' @template param_output_log
 #'
 #' @return Object of class [R6::R6Class] and `Rush` with controller methods.
 #' @export
@@ -146,7 +148,9 @@ Rush = R6::R6Class("Rush",
       packages = NULL,
       lgr_thresholds = NULL,
       lgr_buffer_size = NULL,
-      supervise = TRUE
+      supervise = TRUE,
+      message_log = NULL,
+      output_log = NULL
       ) {
       n_workers = assert_count(rush_env$n_workers %??% n_workers, .var.name = "n_workers")
       lgr_thresholds = assert_vector(rush_env$lgr_thresholds %??% lgr_thresholds, names = "named", null.ok = TRUE, .var.name = "lgr_thresholds")
@@ -169,17 +173,19 @@ Rush = R6::R6Class("Rush",
       config = mlr3misc::discard(unclass(self$config), is.null)
       config = paste(imap(config, function(value, name) sprintf("%s = '%s'", name, value)), collapse = ", ")
       config = paste0("list(", config, ")")
-      lgr_thresholds = paste(imap(lgr_thresholds, function(value, name) sprintf("%s = '%s'", name, value)), collapse = ", ")
+      lgr_thresholds = paste(imap(lgr_thresholds, function(value, name) sprintf("'%s' = '%s'", name, value)), collapse = ", ")
       lgr_thresholds = paste0("c(", lgr_thresholds, ")")
+      message_log = if(is.null(message_log)) "NULL" else sprintf("'%s'", message_log)
+      output_log = if(is.null(output_log)) "NULL" else sprintf("'%s'", output_log)
 
       # generate worker ids
       worker_ids = adjective_animal(n = n_workers)
 
       self$processes_processx = c(self$processes_processx, set_names(map(worker_ids, function(worker_id) {
        processx::process$new("Rscript",
-        args = c("-e", sprintf("rush::start_worker(network_id = '%s', worker_id = '%s', config = %s, remote = FALSE, lgr_thresholds = %s, lgr_buffer_size = %i)",
-          self$network_id, worker_id, config, lgr_thresholds, lgr_buffer_size)),
-        supervise = supervise, stderr = "|", stdout = "|") # ,
+        args = c("-e", sprintf("rush::start_worker(network_id = '%s', worker_id = '%s', config = %s, remote = FALSE, lgr_thresholds = %s, lgr_buffer_size = %i, message_log = %s, output_log = %s)",
+          self$network_id, worker_id, config, lgr_thresholds, lgr_buffer_size, message_log, output_log)),
+        supervise = supervise, stderr = "|")
       }), worker_ids))
 
       return(invisible(worker_ids))
@@ -202,7 +208,9 @@ Rush = R6::R6Class("Rush",
       globals = NULL,
       packages = NULL,
       lgr_thresholds = NULL,
-      lgr_buffer_size = NULL
+      lgr_buffer_size = NULL,
+      message_log = NULL,
+      output_log = NULL
       ) {
       n_workers = assert_count(rush_env$n_workers %??% n_workers, .var.name = "n_workers")
       lgr_thresholds = assert_vector(rush_env$lgr_thresholds %??% lgr_thresholds, names = "named", null.ok = TRUE, .var.name = "lgr_thresholds")
@@ -243,7 +251,9 @@ Rush = R6::R6Class("Rush",
             config = config,
             remote = TRUE,
             lgr_thresholds,
-            lgr_buffer_size)),
+            lgr_buffer_size,
+            message_log = message_log,
+            output_log = output_log)),
         worker_ids))
 
       return(invisible(worker_ids))
@@ -262,7 +272,9 @@ Rush = R6::R6Class("Rush",
       lgr_thresholds = NULL,
       lgr_buffer_size = NULL,
       heartbeat_period = NULL,
-      heartbeat_expire = NULL
+      heartbeat_expire = NULL,
+      message_log = NULL,
+      output_log = NULL
       ) {
       lgr_thresholds = assert_vector(rush_env$lgr_thresholds %??% lgr_thresholds, names = "named", null.ok = TRUE, .var.name = "lgr_thresholds")
       lgr_buffer_size = assert_count(rush_env$lgr_buffer_size %??% lgr_buffer_size %??% 0, .var.name = "lgr_buffer_size")
@@ -319,7 +331,7 @@ Rush = R6::R6Class("Rush",
         }
 
         # stop running workers
-        self$stop_workers(type = "kill", intersect(rush$running_worker_ids, worker_id_mirai))
+        self$stop_workers(type = "kill", intersect(self$running_worker_ids, worker_id_mirai))
 
         lg$info("Restarting %i worker(s): %s", length(worker_id_mirai), str_collapse(worker_id_mirai))
 
@@ -336,7 +348,7 @@ Rush = R6::R6Class("Rush",
               network_id = self$network_id,
               config = config,
               remote = TRUE,
-              lgr_thresholds = c(rush = "debug"))), # FIXME LOG Level
+              lgr_thresholds = c("mlr3/rush" = "debug"))), # FIXME LOG Level
           worker_id_mirai)
 
         self$processes_mirai = insert_named(self$processes_mirai, new_processes)
@@ -565,8 +577,11 @@ Rush = R6::R6Class("Rush",
         if (all(running)) return(invisible(self))
 
         # search for associated worker ids
-        heartbeat_keys = heartbeat_keys[!running]
-        lost_workers = self$worker_info[heartbeat_keys, worker_id, on = "heartbeat"]
+        running_worker_ids = self$running_worker_ids
+        expired_heartbeat_keys = heartbeat_keys[!running]
+        cmds = map(running_worker_ids, function(worker_id) c("HMGET", private$.get_key(worker_id), "heartbeat"))
+        all_heartbeat_keys = unlist(r$pipeline(.commands = cmds))
+        lost_workers = running_worker_ids[all_heartbeat_keys %in% expired_heartbeat_keys]
 
         # set worker state
         cmds = map(lost_workers, function(worker_id) {
@@ -1431,10 +1446,20 @@ Rush = R6::R6Class("Rush",
 
       # fix type
       worker_info[, remote := as.logical(remote)][]
-      worker_info[, heartbeat := as.logical(heartbeat)][]
+      worker_info[, heartbeat := heartbeat != "NA"][]
       worker_info[, pid := as.integer(pid)][]
 
-      worker_info
+      # worker states
+      worker_ids = list(
+        running = data.table(worker_id = self$running_worker_ids),
+        terminated =  data.table(worker_id = self$terminated_worker_ids),
+        killed =  data.table(worker_id = self$killed_worker_ids),
+        lost =  data.table(worker_id = self$lost_worker_ids)
+      )
+
+      worker_states = rbindlist(worker_ids, idcol = "state", use.names = TRUE, fill = TRUE)
+
+      worker_info[worker_states, , on = "worker_id"]
     },
 
     #' @field worker_states ([data.table::data.table()])\cr
