@@ -9,6 +9,8 @@
 #' Process ID of the worker.
 #' @param config ([redux::redis_config])\cr
 #' Redis configuration options.
+#' @param heartbeat_key (`character(1)`)\cr
+#' Heartbeat key.
 #'
 #' @template param_network_id
 #' @template param_worker_id
@@ -18,10 +20,9 @@
 #' @return `NULL`
 #' @keywords internal
 #' @export
-heartbeat = function(network_id, config, worker_id, heartbeat_period, heartbeat_expire, pid) {
+heartbeat = function(network_id, config, worker_id, heartbeat_key, heartbeat_period, heartbeat_expire, pid) {
   r = redux::hiredis(config)
   worker_id_key = sprintf("%s:%s", network_id, worker_id)
-  heartbeat_key = sprintf("%s:%s:heartbeat", network_id, worker_id)
   kill_key = sprintf("%s:%s:kill", network_id, worker_id)
 
   repeat {
